#
#
command = xplor.command
from jCoupPot import JCoupPot 
from noePot import NOEPot
from xplor import select
from xplorPot import XplorPot
from rdcPotTools import *
from pdbTool import *
from atomAction import *
from selectTools import *
from simulationTools import *
from ivm import IVM
import string

simWorld.setRandomSeed( 412 )


command("""
 structure @../Psf/sry_wt.psf 
           @../Psf/axis_new.psf
           @../Psf/sry_dna.psf
           @../Psf/axis_600.psf
           @../Psf/axis_700.psf
 end

 param @../TopPar/parallhdg_new.pro
       @../TopPar/parnah1er1_mod_new.inp
       @../TopPar/par_axis_3.pro 
 end
 set echo = on message = on end
""")

command("""
 coor @../Structures/final_ave.min
 coor copy end
""")

command("""
 collapse
   assign (resid 4:81 or resid 105:112 or resid 117:124) 25.0 15.0
   scale 1.0
 end
""")

#
# NOE
# 
command("""
 noe
   reset
   nres = 5000             
   ceiling 1000
!   set message off echo off end
   class all 
   @../Restraints/6_22noe_prot_fin.tbl
   @../Restraints/6_22noe_prot_fin_new.tbl
   @../Restraints/6_30n15noe_arg.tbl
   @../Restraints/7_2metonly.tbl
   @../Restraints/helix_hbond.tbl 
   @../Restraints/dna_trial1.tbl !@dna_final.tbl !@noe_dna_rx_trial.tbl 
   @../Restraints/endnoe_dnax.tbl
   @../Restraints/pp_trial.tbl !@pp_trialx.tbl !@pp_trial.tbl
   @../Restraints/trial_inter3.tbl !@class_inter.tbl
   @../Restraints/6_22nh_inter.tbl
   @../Restraints/inter_hb.tbl 
   @../Restraints/ends_bdnazzz.tbl 

  averaging  all sum
  potential  all square   !soft
  scale      all 30.0 
  sqconstant all 1.0
  sqexponent all 2
end
""")

command("""
noe
   class dnah
!   @../Restraints/hbonds_dna.tbl
   @../Restraints/hbonds_dna-new.tbl
!   @../Restraints/hbonds_dna_edta.tbl
   averaging dnah sum
   potential dnah biharm
   scale dnah 10.0
   sqexponent dnah 2
   sqconstant dnah 1.0
end
""")

command("""
 carbon
  phistep=180
  psistep=180
  nres=300
  class all
  force 0.5
  potential harmonic
  @../TopPar/rcoil_c13.tbl             !rcoil shifts
  @../TopPar/expected_edited_c13.tbl   !13C shift database
  !set echo off message off end
  @../Restraints/c13_shifts.tbl        !carbon shifts
  !set echo on message on end
 end
""")

#
# repulsive restraints for EDTA-DNA phosphate
#
#rep = NOEPot("rep", open("../Restraints/repul-EDTA-Phosphate.tbl").read() )
#rep.setPotType( "hard" )
#rep.setAveType( "sum"  )
#rep.setScale(30.0)

#
# Dihedral angle restraints
#
command("""
 restraints dihed 
  reset
  scale 200.0  
  nass = 10000
!  set message off echo off end
  @../Restraints/dihed_final.tbl   !has some psi from GC/families
  @../Restraints/dihed_dna_final.tbl  
  @../Restraints/chi_final-altered.tbl
  !set message on echo on end
 end 
""")

command("""
 couplings
   nres 400
   potential harmonic
  class phi
   degen 1
   force 1.0
   !set echo off message off end
   coefficients 6.98 -1.38 1.72 -60.0 
     @../Restraints/hnha_xplr.tbl
 { class gly  !for gly where you don't know stereoassignement
   degen 2
   force 1.0 0.2
   coefficients 6.98 -1.38 1.72 -60.0
      @../phij_gly_final.tbl     
   !set echo on message on end
 }
  class e
   degen 1
   force 0.0
   coefficients 15.3 -6.2 1.5 120.0 !15.3cos2(e+120)-6.2cos(e+120)+1.5
      @../Restraints/dna_pcoup.tbl
   !set echo on message on end
 end
""")

#
#
#
command("""
 sani
   nres=1000
   class JNH
   force 1.0   
   potential harmonic
   coeff 0.0 8.6 0.610
   @../Restraints/dip_nh.tbl

   class JCH
   force 1.0      
   potential harmonic
   coeff 0.0 8.6 0.610
   @../Restraints/dip_caha.tbl

!   class JCACO
!   force $ksani_CACO
!   potential harmonic
!   coeff 0.0 8.6 0.610
!   @dip_caco.tbl

   class JNCO
   force 0.050     
   potential harmonic
   coeff 0.0 8.6 0.610
   @../Restraints/dip_nc.tbl

   class JHNC
   force 0.108      
   potential harmonic
   coeff 0.0 8.6 0.610
   @../Restraints/dip_hnc.tbl

   class JNHp
   force 1.0         
   potential harmonic
   coeff 0.0 8.5 0.60
   @../Restraints/17dip_protnh.tbl

   class JNHd
   force 1.0
   potential harmonic
   coeff 0.0 8.5 0.60
   !@17dip_nh.tbl
   @../Restraints/junk_dna_nh.tbl

   class JCHd
   force 1.0           
   potential harmonic
   coeff 0.0 8.5 0.60
   !@17dip_aro.tbl
   !@17dip_sug.tbl
   !@junk_dna_sug.tbl
  ! @../Restraints/final_dna_sug.tbl
   @../Restraints/final_dna_sug-new.tbl ! modified version of final_dna_sug.tbl
   @../Restraints/final_dna_aro.tbl

 end
""")

command("""
 xdip
   nres=5000
   class JCHg                    ! ribose
        type fixd
        scale 1.0
        sign on
        average sum
        force 1.0   
        potential harmonic
        coeff 0.0 8.0 0.59
        @../Restraints/CAH_gabriel.tbl

   class HHp
        type vard
        scale 10.46847258
        sign off
        average average
        force 1.0   
        potential square
        coeff 0.0 8.0 0.59
        @../Restraints/wt_HH_dipy.tbl

   class HHn
        type vard
        scale 10.46847258
        sign off
        average sum
        force 0.0 !$ksani
        potential square
        coeff 0.0 8.0 0.59
        @../Restraints/HH_dip_negx.tbl
 end
""")

#
# setup of base-base positional database
#
command("""
  !quartic 
  @bbpd.setup

  !gaussian
  !@bbpdGauss.setup
""")

#
# setup of nucleic acid torsion angle database
#
command("""
  rama
    nres=10000
    @../TopPar/shortrange_gaussians.tbl

    ! with quartic bell for nucleic acid
    !@../TopPar/nucleic_deltor_quarts2d.tbl
    !@../TopPar/nucleic_deltor_quarts3d.tbl
    !@../TopPar/nucleic_deltor_quarts4d.tbl

    ! with gaussian bell for nucleic acid
    @../TopPar/all_na_gaussians.tbl

  end
  @../TopPar/newshortrange_setup.tbl

  ! with quartic bell for nucleic acid
  !@../TopPar/setup_nucleic_2d3d.tbl
  !@../TopPar/setup_nucleic_4d.tbl

  ! with gaussian bell for nucleic acid
  @../TopPar/setup_nucleic_new.tbl
""")

tmps = open("../TopPar/proShortRange.list").read() 
proShortRange = tmps.split(None)
tmps = open("../TopPar/nuc2d.listA").read()       
nuc2dA  = tmps.split(None)
tmps = open("../TopPar/nuc2d.listB").read()       
nuc2dB  = tmps.split(None)
tmps = open("../TopPar/nuc3d4d.list").read()      
nuc3d4d = tmps.split(None)

tmps = open("../TopPar/nucleic.list").read()
nucleic = tmps.split(None)


#
# setup of planerity restraints
#

command("""
restraints plane
 @../Restraints/dnaPlanBp_14bp.tbl
end
""")

#
# setup parameters for nbond term.
#

command("""
   parameters  ! just repulsive term
    nbonds
      atom
      nbxmod 3  wmin 0.01 cutnb 4.5 tolerance 0.5
      rexp 2 irex 2 ctonnb=2.99 ctofnb=3.
      repel= 0.5 rcon= 1.0 ! changed later
    end
   end
""")

#
# annealing settings
#

ini_rad  = 0.9  ; fin_rad = 0.78
ini_con = 0.004 ; fin_con = 4.0
ini_ang = 0.4   ; fin_ang = 1.0
ini_imp = 0.1   ; fin_imp = 1.0

ini_rdc  =  0.01  ; fin_rdc  =  1.0
ini_noe  =  2.0   ; fin_noe  = 30.0
ini_rama =  0.002 ; fin_rama =  1.0
ini_nuc  =  1.00  ; fin_nuc  =  1.0

hot_cdi =  10.0   ; cool_cdi = 200.0

hitemp_potList = PotList()
xpTerms = ["BOND", "ANGL", "IMPR", "VDW", "RAMA", "ORIE", "PLAN", "NOE", "CDIH", "SANI", "XDIP", "CARB", "COUP", "COLL"] 
for xn in  xpTerms :
   hitemp_potList.add( XplorPot(xn) )
   pass


#set mass and fric
AtomSel("not (resname ANI or resname TAU)").apply(SetProperty("mass", 100.0))
AtomSel("resname ANI or resname TAU ").apply(SetProperty("mass", 300.0))
AtomSel("all").apply(SetProperty("fric", 10.0))

#
# IVM setup
#

dyn  = IVM() 
dyn.reset() ; dyn.potList().removeAll()

#tedBB = ['P', 'O1P', 'O2P', 'C1\'', 'H1\'', 'C2\'', 'H2\'', 'H2\'\'', 'C3\'', 'H3\'', 'O3\'', 'C4\'', 'H4\'', 'O4\'', 'C5\'', 'H5\'', 'H5\'\'', 'O5\'']
#tedBase = ['C6', 'H6', 'N3', 'H3', 'C2', 'O2', 'N1', 'C5', 'C4', 'O4']

#
# common setting of groups and hinges
#
groupList  = dyn.groupList()   # these should be after setting fixed atoms
hingeList  = dyn.hingeList()   #

# group EDTA-Mn
for res in ([ (215, 300), (129, 301), (103, 302)]) :
   for i in range(0,10) :
      groupList.append(select('segid ALT%-d and ((resid %d and (name OY* or name NY* or name HY* or name CY*)) or resid %d)' % (i, res[0], res[1])))
      pass
   pass

# group axes for RDC
for ax in (500, 600, 700):
   groupList.append(select('resid %d' % ax))
   hingeList.append( ("rotate", select('resid %d' % ax) ) )
   pass

#break ribose rings
IVM_breakRiboses(dyn)

#break proline rings
IVM_breakProlines(dyn)
    
dyn.setGroupList( groupList )
dyn.setHingeList( hingeList )

IVM_groupRigidSidechain(dyn)
dyn.autoTorsion()

print groupList
print hingeList

#dyn.setVerbose(dyn.printNodeDef)

def setConstraints(k_ang,k_imp):
    #number of conformers (support up to 10 conformers)
    nconf = 3
    scaleVdw = 1.0 / float( nconf )
    command("""
      constraints 
      !   interaction (all) (all) 
         inter = (segid " ") (segid " ")
         inter = (segid "ALT1") (segid "ALT1")
         inter = (segid "ALT2") (segid "ALT2")
         inter = (segid "ALT3") (segid "ALT3")
         inter = (segid "ALT4") (segid "ALT4")
         inter = (segid "ALT5") (segid "ALT5")
         inter = (segid "ALT6") (segid "ALT6")
         inter = (segid "ALT7") (segid "ALT7")
         inter = (segid "ALT8") (segid "ALT8")
         inter = (segid "ALT9") (segid "ALT9")
         inter = (segid "ALT0") (segid "ALT0")
         weights * 1 angl %f impr %f end 
         inter = (segid " ") (segid "ALT1")
         inter = (segid " ") (segid "ALT2")
         inter = (segid " ") (segid "ALT3")
         inter = (segid " ") (segid "ALT4")
         inter = (segid " ") (segid "ALT5")
         inter = (segid " ") (segid "ALT6")
         inter = (segid " ") (segid "ALT7")
         inter = (segid " ") (segid "ALT8")
         inter = (segid " ") (segid "ALT9")
         inter = (segid " ") (segid "ALT0")
         weights * 1 angl %f impr %f vdw %f end 
      end""" % ( k_ang, k_imp, k_ang, k_imp, scaleVdw) )
    return

def checkAndCorrect(dyn, pene, ene, tmstp, ni, maxEner):
    # successful or not?
    #
    badIter = 0
    if ene > maxEner :
      badIter = 1
      print "energy is too high (%7.1f)" % ene
    if ene > pene and ene/pene > 10.0 :
      badIter = 1
      print "energy is higher than ten times of previous value (current: %7.1f, previous: %7.1f)" % (ene, pene)
    if tmstp < 5.0e-4 :
      print "timestep is too short (%e ps)" % tmstp
      badIter = 1

    #optional
    #command("""
    #   print thres=0.03 bonds
    #   print thres=5. angles
    #   print thres=5. impropers
    #""")

    # for successful case
    #  - copy the coordinates to comparison set
    #
    if badIter == 0 : 
      command("""
        coor copy end ! copy good coordinates
      """)
      return 1
   
    # for unsuccessful case
    #  - copy back the previous coordinates  
    #  - randomize initial velocities

    command("""
      coor swap end ! call previous good coordinates
      coor copy end ! <- to erase bad one
    """)

    if ni > 3 :
       print "Too many corrections. Exiting programs"
       exit(1)

    bath = dyn.bathTemp()
    randomizeVelocities( bath )
    dyn.resetReuse()  #necessary to get randomized velocities
    return 0

def dynamicsWithCheck(s, timestep, bath, finalTime, maxcycles):
    s.calcEnergy()
    ok = 0
    ni = 0
    while ok == 0 :
         pene   = s.Epotential()
         s.setNumSteps(maxcycles)
         s.setStepsize( timestep )
         s.setBathTemp( bath )
         s.setFinalTime( finalTime )
         s.setPrintInterval( 10 )
         s.setETolerance( bath/1000 )
         s.setResetCMInterval(10)
         s.setResponseTime(20)
         #dyn.setVerbose(0xffff)
         dyn.setMaxDeltaE(10000)
         #dyn.setAdjustStepsize(0)
         s.run()
         ene    = s.Epotential()
         tmstp  = s.stepsize()
         ok = checkAndCorrect(s, pene, ene, tmstp, ni, 100000.0)
         ni = ni + 1
         pass
    return


def setNoeForce(k_noe):
    command("noe scale all %f end" % k_noe)
    print "(setNoeForce) current k_noe: ", k_noe
    return

def setRdcForce(k_rdc):
    ksani        = k_rdc * 1.0
    ksani_CH     = k_rdc * 1.0
    ksani_NCO    = k_rdc * 0.050
    ksani_HNC    = k_rdc * 0.108
    ksani_prot   = k_rdc * 1.0
    ksani_dna    = k_rdc * 1.0
    ksani_CH_dna = k_rdc * 1.0
    command("sani class JNH  force %f end " % ksani       )
    command("sani class JNCO force %f end " % ksani_NCO   )
    command("sani class JCH  force %f end " % ksani_CH    )
    command("sani class JHNC force %f end " % ksani_HNC   )
    command("sani class JNHp force %f end " % ksani_prot  )
    command("sani class JNHd force %f end " % ksani_dna   )
    command("sani class JCHd force %f end " % ksani_CH_dna)
    command("xdip class JCHg force %f end " % ksani       )
    command("xdip class HHp  force %f end " % ksani       )
    command("xdip class HHn  force 0.0 end")
    print "(setRdcForce) current ksani: ", ksani
    return

def setProRamaForce(k_rama):
    trace.suspend()
    for cn in (proShortRange):
       command("rama class %s force %f end " % (cn, k_rama))
       pass
    trace.resume()
    print "(setProRamaForce) current k_rama: ", k_rama
    return

def setNucRamaForce(k_nuc):
 
    # for quartic bell
    #trace.suspend()
    #2D containing non-delta 
    #k2d=0.0
    #for cn in (nuc2dA):
    #   command("rama class %s force %f end " % (cn, k2d))
    #   pass
    #2D containing delta
    #kdelta=0.0
    #for cn in (nuc2dB):
    #   command("rama class %s force %f end " % (cn, kdelta))
    #   pass
    #3D & 4D
    #for cn in (nuc3d4d):
    #   command("rama class %s force %f end " % (cn, k_nuc))
    #   pass
    #trace.resume()

    #for gaussian
    trace.suspend()
    for cn in (nucleic):
       command("rama class %s force %f end " % (cn, k_nuc))
       pass
    trace.resume()
    print "(setNucRamaForce) current k_nuc: ", k_nuc
    return

def structLoopAction(loopInfo):

    #
    # read initial structure
    #
    command("""
     coor init end
     coor @../Structures/final_ave.min
     coor copy end
    """)

    dyn.init()
    #
    # regularization of covalent geometry
    #                       
    command("""
      flags exclude * include bond angl impr end
      mini powell nstep 1000 nprint 50 end
    """)

    #
    # high temp dynamics with vdw 
    #

    # potential terms to use
    dyn.setPotList( hitemp_potList)

    # set some high-temp force constants
    setNoeForce( ini_noe )
    setRdcForce( ini_rdc )
    setProRamaForce( ini_rama )
    setNucRamaForce( ini_nuc  )
    command("restraints dihed scale %f end" % hot_cdi )

    # VDW setting for high temperature
    command("""
      parameters
        nbonds
          atom 
          nbxmod 3 
          wmin  =   0.01  cutnb =   100   tolerance 45 
          repel= 1.2   rexp   =  2   irex   =  2  rcon=1.0  
        end
      end 
      constraints 
       inter = (not (name ca or name p or name mn)) (all) 
       weights * 1 angl 0.4 impr 0.1 vdw 0 elec 0 end 
       inter = (name ca or name p or name mn) (name ca or name p or name mn) 
       weights * 1 angl 0.4 impr 0.1 vdw 1.0 end 
      end
    """)


    # dynamics
    timestep  = 0.002
    bath = 3000.0
    dyn.setStepType("pc6")
    randomizeVelocities( bath )
    dyn.resetReuse()  #necessary to get randomized velocities
    dyn.calcEnergy()
    for i_high in range(0,10) :
       print " high temp dynamics -- subcycle No. %d" % i_high
       dynamicsWithCheck(dyn, timestep, bath, 1.0, 500)
       pass


    #
    # cooling
    #

    #number of cycles
    numSteps = 240

    global k_ang, k_imp

    # VDW setting for cooling
    command("""
     parameters
      nbonds
       atom
       nbxmod 3  
       wmin 0.01  
       cutnb 4.5  
       tolerance 0.5
       rexp 2 irex 2
       ctonnb=2.99 ctofnb=3. 
       repel=0.5  ! changed later
       rcon=1.0   ! changed later
      end
     end 
    """)
    setConstraints(ini_ang, ini_imp)

    # setting force constants and etc.
    k_rdc       = MultRamp(ini_rdc,fin_rdc,"setRdcForce( VALUE )")
    k_noe       = MultRamp(ini_noe,fin_noe,"setNoeForce( VALUE )")
    k_rama = MultRamp(ini_rama,fin_rama,"setProRamaForce( VALUE )")
    k_nuc  = MultRamp(ini_nuc,fin_nuc,  "setNucRamaForce( VALUE )")
    k_ang  = MultRamp(ini_ang,fin_ang)
    k_imp  = MultRamp(ini_imp,fin_imp)
    radius = MultRamp(ini_rad,fin_rad, "command('param nbonds repel VALUE end end')")
    k_vdw  = MultRamp(ini_con,fin_con, "command('param nbonds rcon VALUE end end')")

    # IMPORTANT LIST
    rampedParams = [k_noe,k_rdc,k_rama,k_nuc,radius,k_vdw,k_ang,k_imp]


    for k in rampedParams :
       k.init(numSteps)
       pass
    command("restraints dihed scale %f end" % cool_cdi )

    #dynamics
    initTemp  = 3000.0
    finalTemp =   25.0
    coolTimestep = 0.002
    tempStep  = (initTemp-finalTemp)/float(numSteps)
    finalTime = 0.2
    maxCycle =  100
    bathTemp = initTemp;
    for i_cool in range(0,numSteps):
       bathTemp -= tempStep
       for k in rampedParams :
            k.update()
            pass
       setConstraints(k_ang.value(), k_imp.value())
       print "cooling cycle - No.%d-" % i_cool
       print "current bathTemp = %7.2f" % bathTemp
       dynamicsWithCheck(dyn, coolTimestep, bathTemp, finalTime, maxCycle)
       pass

    #
    # Powell minimization in torsion angle space
    #
    dyn.setStepType("powell")
    dyn.setNumSteps(20000)
    dyn.setMaxCalls(20000)
    dyn.setETolerance(1.0e-7)
    dyn.setGTolerance(0.01)
    dyn.setDEpred(1.0)
    dyn.run()

    #
    # Powell minimization in Cartesian space
    #
    command("""
    flags exclude * 
          include bond angle impr vdw rama orie plan noe cdih coup carb sani xdip coll
    end
    mini powell nstep 2000 nprint 10 end
    """)

    dyn.resetReuse()
    dyn.init()
    dyn.calcEnergy()

    outFile = PDBTool("../Structures/saNoConju%d.pdb" % (loopInfo.count+1))

    en = {}
    eAll  = dyn.Epotential()
    for xn in xpTerms :
       en[xn] = XplorPot(xn).calcEnergy().energy
       pass

    rms = {}
    violations = {}
    for (term, threshold) in (("BOND",0.05),
                              ("ANGL", 5.0),
                              ("IMPR", 5.0),
                              ("NOE",  0.5),
                              ("CDIH", 5.0)):
        result = command("print thresh %f %s"% (threshold,term),
                         ("RESULT","VIOLATIONS"))
        rms[term] = float(result[0])
        violations[term] = int(result[1])
        pass

    result =  command("carbon print threshold = 1.0 end", ("RMSCA","RMSCB"))
    rms["CASHIFT"] = float(result[0])
    rms["CBSHIFT"] = float(result[1])

    result =  command("couplings print threshold 1.0 class phi end", ("RESULT","VIOLATIONS"))
    rms["COUP"] = float(result[0])

    result =  command("xdip print threshold 0.0 class HHp end", ("RESULT","VIOLATIONS"))
    rms["XDIP"] = float(result[0])



    outFile.addRemark("OVERALL ENERGY: %7.2f " % eAll)
    outFile.addRemark("BOND: %7.2f, ANGL: %7.2f, IMPR: %7.2f, VDW: %7.2f" % (en["BOND"], en["ANGL"], en["IMPR"], en["VDW"]))

    outFile.addRemark("RAMA: %7.1f, ORIE: %7.1f, PLAN: %5.1f, COLL: %5.1f" % (en["RAMA"], en["ORIE"], en["PLAN"], en["COLL"]) )
    outFile.addRemark("NOE:  %7.2f, CDIH: %7.2f" % (en["NOE"] , en["CDIH"]))
    outFile.addRemark("COUP: %7.2f, CARB: %7.2f" % (en["COUP"], en["CARB"]))
    outFile.addRemark("SANI: %7.2f, XDIP: %7.2f" % (en["SANI"], en["XDIP"]))
    bars = "----------------------------------------------------------"
    outFile.addRemark(bars)
    outFile.addRemark("RMS")
    outFile.addRemark("bond: %7.4f, angl: %7.4f, impr: %7.4f" % (rms["BOND"], rms["ANGL"], rms["IMPR"]))
    outFile.addRemark("noe:  %7.4f, cdih: %7.4f, coup: %7.4f" % (rms["NOE"], rms["CDIH"], rms["COUP"]))
    outFile.addRemark("13CA shift: %6.3f, 13CB shift: %6.3f, XDIP HHp: %6.3f" % (rms["CASHIFT"], rms["CBSHIFT"], rms["XDIP"]))
    outFile.addRemark("-violation- noe:%d, cdih:%d" % (violations["NOE"], violations["CDIH"]))
    
    outFile.addRemark(bars)
    outFile.addRemark("sani          rms,  R-factor")
    for (clas, denomi)  in (("JNH", 12.3 ),
                            ("JNCO",12.3 ),
                            ("JCH", 12.3 ),
                            ("JHNC",12.3 ),
                            ("JNHd",12.11),
                            ("JCHd",12.11)):
        result = command("sani print threshold=0.0 class %s end" % clas,
                         ("RESULT","VIOLATIONS"))
        rrms = float(result[0])
        rf = rrms / denomi
        outFile.addRemark("%9s : %7.3f %7.3f" % (clas, rrms, rf))
        pass


    outFile.addRemark(bars)
    outFile.write()
    pass

StructureLoop(numStructures=64,
              structLoopAction=structLoopAction).run()
