#
#
#
#

command = xplor.command
from jCoupPot import JCoupPot 
from noePot import NOEPot

import prePot

from xplor import select

from xplorPot import XplorPot
from rdcPotTools import *
from pdbTool import *
from atomAction import *
from selectTools import *
from simulationTools import *
from ivm import IVM

command("""
 structure @../Psf/totSitesABonly1Alt.psf end
 param @../TopPar/parallhdg_new.pro
       @../TopPar/parnah1er1_mod_new.inp
       @../TopPar/par_axis_3.pro 
       @../TopPar/edtaThymine.par
       @../TopPar/manganese.par
       @../TopPar/parClockAtoms.inp
 end
 @../TopPar/patchForMn-EDTA.inp
 set echo = on message = on end
""")

command("""
 coor @../Structures/iniSitesABonly1Alts.pdb end
""")

command("""
parameters
   nbonds
      atom
      repel=0.5                 {*Initial value for effective atom radius  *}
                                {*--modified later.*}
      nbxmod 4  {* Can use 4 here, due to internal coordinate dynamics. *}
      wmin 0.01  
      cutnb 4.5  
      tolerance 0.5
      rexp 2 irex 2
      rcon=1.
      ctonnb=2.99 ctofnb=3. 
   end
end""")


#
# annealing settings
#

init_t  = 3000.
final_t = 25


cool_steps = 12000

ini_rad  = 0.4 
ini_con = 0.004
ini_ang = 0.4
ini_imp = 0.4     # was 0.1

hitemp_potList = PotList()
hitemp_potList.add( XplorPot("BOND") )
hitemp_potList.add( XplorPot("ANGL") )
hitemp_potList.add( XplorPot("IMPR") )

for atom in AtomSel("all"):
    atom.setMass(100.)
    atom.setFric(10.)
    pass

#
# IVM setup
#

dyn = IVM()

#
# reset ivm for torsion-angle dynamics
#
dyn.reset()
dyn.potList().removeAll()

dyn.fix( select('resid 1:85') )
dyn.fix( select('resid 99:102') )     
dyn.fix( select('resid 104:128') ) 
dyn.fix( select('resid 130') )
dyn.fix( select('resid 213:214') )     
dyn.fix( select('resid 216') )     
dyn.fix( select('resid 500:800') )

tedBB = ['P', 'O1P', 'O2P', 'C1\'', 'H1\'', 'C2\'', 'H2\'', 'H2\'\'', 'C3\'', 'H3\'', 'O3\'', 'C4\'', 'H4\'', 'O4\'', 'C5\'', 'H5\'', 'H5\'\'']

for a1 in tedBB :
   dyn.fix( select('(resid 129 or resid 215 or resid 103) and name %s' % a1))
   pass

tedBase = ['C6', 'H6', 'N3', 'H3', 'C2', 'O2', 'N1', 'C5', 'C4', 'O4']
for a1 in tedBase :
   dyn.fix( select('(resid 129 or resid 215 or resid 103) and name %s' % a1))
   pass

groupList = dyn.groupList()

#
# grouping individual EDTA-MN as rigid bodies
#
# & deleting bond lists between different conformers
#

for res in ([ (300, 215), (301, 129), (302, 103)]) : 
   for i in range(0,10) :
      groupList.append(select('segid ALT%-d and ((resid %d and (name OY* or name NY* or name HY* or name CY*)) or resid %d)' % (i, res[1], res[0])))
      for id in  AtomSel("segid ALT%-d and (resid %d or resid %d)" % (i, res[0], res[1])).indices() :
         for j in range(i+1,10) :
            if i == j : continue
            for id2 in AtomSel("segid ALT%-d and bondedto id %d" % (j, id+1)).indices() :
                dyn.breakBond( (id, id2) )
                print "deleting bond between %d and %d " %  (id, id2)
                pass
            pass
         pass
      pass
   pass

dyn.setGroupList( groupList )


IVM_groupRigidSidechain(dyn)
dyn.setStepType("pc6")
dyn.autoTorsion()

def setConstraints(k_ang,k_imp):
    command("""
      constraints 
      !   interaction (all) (all) 
         inter = (segid " ") (segid " ")
         inter = (segid " " or segid "ALT1") (segid "ALT1")
         inter = (segid " " or segid "ALT2") (segid "ALT2")
         inter = (segid " " or segid "ALT3") (segid "ALT3")
         inter = (segid " " or segid "ALT4") (segid "ALT4")
         inter = (segid " " or segid "ALT5") (segid "ALT5")
         inter = (segid " " or segid "ALT6") (segid "ALT6")
         inter = (segid " " or segid "ALT7") (segid "ALT7")
         inter = (segid " " or segid "ALT8") (segid "ALT8")
         inter = (segid " " or segid "ALT9") (segid "ALT9")
         inter = (segid " " or segid "ALT0") (segid "ALT0")
            weights * 1 angl %f impr %f
         end 
      end""" % ( k_ang, k_imp) )
    return


def structLoopAction(loopInfo):

    #
    # potential terms to use
    #
    
    dyn.setPotList( hitemp_potList)

    #
    # set some high-temp force constants
    #


    command("parameters  nbonds repel %f end end" % ini_rad)
    command("parameters  nbonds rcon %f end end" % ini_con)
    setConstraints(ini_ang, ini_imp)

    #
    # high temp dynamics
    #

    ini_timestep = 0.010
    bath = init_t
    timestep = ini_timestep
    randomizeVelocities( bath )
    dyn.setNumSteps(0)
    dyn.setFinalTime(50)
    dyn.setStepsize( timestep )
    dyn.setBathTemp( bath )
    dyn.setPrintInterval( 100 )
    dyn.setETolerance( bath/1000 )
#    dyn.setVerbose(0xffff)
    dyn.resetReuse()  #necessary to get randomized velocities
    dyn.run()

    #
    # cooling
    #

    global k_ang, k_imp
    radius = MultRamp(ini_rad,0.8,
                      "command('param nbonds repel VALUE end end')")
    k_vdw  = MultRamp(ini_con,4, "command('param nbonds rcon VALUE end end')")
    k_ang  = MultRamp(ini_ang,1.0)
    k_imp  = MultRamp(ini_imp,1.0)

    dyn.potList().add( XplorPot("VDW") )
    dyn.setStepsize( timestep )
    dyn.setFinalTime( 0.5 )
    dyn.setPrintInterval( 100 )
    dyn.setResetCMInterval( 100 )

    AnnealIVM(initTemp =init_t,
              finalTemp=final_t,
              numSteps = 50,
              ivm=dyn,
              rampedParams = [radius,k_vdw,k_ang,k_imp],
              extraCommands= lambda notUsed: \
                 setConstraints(k_ang.value(),
                                k_imp.value())).run()
              
    #
    # final Powell minimization
    #
    #dyn.setStepType("powell")
    #dyn.run()

    #command("mini powell nstep 500 nprint 50 end")
    dyn.resetReuse()
    dyn.init()
    dyn.calcEnergy()

    #  !
    #  ! recenter
    #  !
    #
    #  coor orient end

    #
    # analysis
    # evaluates RMS deviations and violations

    outFile = PDBTool("../Structures/randomABonly1Alts%d.pdb" % (loopInfo.count+1))
    outFile.write()
    pass


StructureLoop(numStructures=3,
              structLoopAction=structLoopAction).run()
